@@ -1,9 +1,99 @@
-use pir_8_emu::isa::instruction::{AluOperationShiftOrRotateDirection, AluOperationShiftOrRotateType, InstructionJumpCondition, InstructionPortDirection,
-                                  InstructionMadrDirection, InstructionStckDirection, InstructionRegisterPair, AluOperation, Instruction};
+use pir_8_emu::isa::instruction::{InstructionLoadImmediateWideRegisterPair, AluOperationShiftOrRotateDirection, AluOperationShiftOrRotateType,
+                                  InstructionJumpCondition, InstructionPortDirection, InstructionMadrDirection, InstructionStckDirection,
+                                  InstructionRegisterPair, AluOperation, Instruction};
 use pir_8_emu::isa::GeneralPurposeRegister;
 use self::super::alt_gp_registers;
 
 
+#[test]
+fn load_immediate_byte() {
+    single_register("LOAD IMM BYTE", |r| Instruction::LoadImmediateByte { rrr: r });
+}
+
+#[test]
+fn load_indirect() {
+    single_register("LOAD IND", |r| Instruction::LoadIndirect { rrr: r });
+}
+
+#[test]
+fn load_immediate_wide() {
+    for regs in &[GeneralPurposeRegister::defaults(), alt_gp_registers()] {
+        assert_eq!(Instruction::LoadImmediateWide { rr: InstructionLoadImmediateWideRegisterPair::Ab }.display(regs).to_string(),
+                   "LOAD IMM WIDE A&B");
+        assert_eq!(Instruction::LoadImmediateWide { rr: InstructionLoadImmediateWideRegisterPair::Cd }.display(regs).to_string(),
+                   "LOAD IMM WIDE C&D");
+        assert_eq!(Instruction::LoadImmediateWide { rr: InstructionLoadImmediateWideRegisterPair::Xy }.display(regs).to_string(),
+                   "LOAD IMM WIDE X&Y");
+        assert_eq!(Instruction::LoadImmediateWide { rr: InstructionLoadImmediateWideRegisterPair::Adr }.display(regs).to_string(),
+                   "LOAD IMM WIDE ADR");
+    }
+}
+
+#[test]
+fn jump() {
+    for regs in &[GeneralPurposeRegister::defaults(), alt_gp_registers()] {
+        assert_eq!(Instruction::Jump(InstructionJumpCondition::Jmpz).display(regs).to_string(), "JMPZ");
+        assert_eq!(Instruction::Jump(InstructionJumpCondition::Jmpp).display(regs).to_string(), "JMPP");
+        assert_eq!(Instruction::Jump(InstructionJumpCondition::Jmpg).display(regs).to_string(), "JMPG");
+        assert_eq!(Instruction::Jump(InstructionJumpCondition::Jmpc).display(regs).to_string(), "JMPC");
+        assert_eq!(Instruction::Jump(InstructionJumpCondition::Jmzg).display(regs).to_string(), "JMZG");
+        assert_eq!(Instruction::Jump(InstructionJumpCondition::Jmzl).display(regs).to_string(), "JMZL");
+        assert_eq!(Instruction::Jump(InstructionJumpCondition::Jmpl).display(regs).to_string(), "JMPL");
+        assert_eq!(Instruction::Jump(InstructionJumpCondition::Jump).display(regs).to_string(), "JUMP");
+    }
+}
+
+#[test]
+fn save() {
+    single_register("SAVE", |r| Instruction::Save { rrr: r });
+}
+
+#[test]
+fn alu_valid() {
+    for regs in &[GeneralPurposeRegister::defaults(), alt_gp_registers()] {
+        assert_eq!(Instruction::Alu(AluOperation::Add).display(regs).to_string(), "ALU ADD");
+        assert_eq!(Instruction::Alu(AluOperation::Sub).display(regs).to_string(), "ALU SUB");
+        assert_eq!(Instruction::Alu(AluOperation::AddC).display(regs).to_string(), "ALU ADDC");
+        assert_eq!(Instruction::Alu(AluOperation::SubC).display(regs).to_string(), "ALU SUBC");
+        assert_eq!(Instruction::Alu(AluOperation::Or).display(regs).to_string(), "ALU OR");
+        assert_eq!(Instruction::Alu(AluOperation::Xor).display(regs).to_string(), "ALU XOR");
+        assert_eq!(Instruction::Alu(AluOperation::And).display(regs).to_string(), "ALU AND");
+        assert_eq!(Instruction::Alu(AluOperation::Not).display(regs).to_string(), "ALU NOT");
+    }
+}
+
+#[test]
+fn alu_valid_shift_or_rotate() {
+    for regs in &[GeneralPurposeRegister::defaults(), alt_gp_registers()] {
+        for &d in &[AluOperationShiftOrRotateDirection::Right, AluOperationShiftOrRotateDirection::Left] {
+            for &tt in &[AluOperationShiftOrRotateType::Lsf,
+                         AluOperationShiftOrRotateType::Asf,
+                         AluOperationShiftOrRotateType::Rtc,
+                         AluOperationShiftOrRotateType::Rtw] {
+                assert_eq!(Instruction::Alu(AluOperation::ShiftOrRotate { d: d, tt: tt }).display(regs).to_string(),
+                           format!("ALU SOR {} {}", d, tt));
+            }
+        }
+    }
+}
+
+#[test]
+fn move_() {
+    for regs in &[GeneralPurposeRegister::defaults(), alt_gp_registers()] {
+        for qqq in regs {
+            for rrr in regs {
+                assert_eq!(Instruction::Move {
+                                   qqq: qqq.address(),
+                                   rrr: rrr.address(),
+                               }
+                               .display(regs)
+                               .to_string(),
+                           format!("MOVE {} {}", qqq.letter(), rrr.letter()));
+            }
+        }
+    }
+}
+
 #[test]
 fn madr() {
     for regs in &[GeneralPurposeRegister::defaults(), alt_gp_registers()] {
@@ -41,100 +131,25 @@ fn madr() {
     }
 }
 
-#[test]
-fn jump() {
-    for regs in &[GeneralPurposeRegister::defaults(), alt_gp_registers()] {
-        assert_eq!(Instruction::Jump(InstructionJumpCondition::Jmpz).display(regs).to_string(), "JMPZ");
-        assert_eq!(Instruction::Jump(InstructionJumpCondition::Jmpp).display(regs).to_string(), "JMPP");
-        assert_eq!(Instruction::Jump(InstructionJumpCondition::Jmpg).display(regs).to_string(), "JMPG");
-        assert_eq!(Instruction::Jump(InstructionJumpCondition::Jmpc).display(regs).to_string(), "JMPC");
-        assert_eq!(Instruction::Jump(InstructionJumpCondition::Jmzg).display(regs).to_string(), "JMZG");
-        assert_eq!(Instruction::Jump(InstructionJumpCondition::Jmzl).display(regs).to_string(), "JMZL");
-        assert_eq!(Instruction::Jump(InstructionJumpCondition::Jmpl).display(regs).to_string(), "JMPL");
-        assert_eq!(Instruction::Jump(InstructionJumpCondition::Jump).display(regs).to_string(), "JUMP");
-    }
-}
-
-#[test]
-fn load_immediate() {
-    single_register("LOAD IMM", |r| Instruction::LoadImmediate { aaa: r });
-}
-
-#[test]
-fn load_indirect() {
-    single_register("LOAD IND", |r| Instruction::LoadIndirect { aaa: r });
-}
-
-#[test]
-fn save() {
-    single_register("SAVE", |r| Instruction::Save { aaa: r });
-}
-
-#[test]
-fn alu_valid() {
-    for regs in &[GeneralPurposeRegister::defaults(), alt_gp_registers()] {
-        assert_eq!(Instruction::Alu(AluOperation::Add).display(regs).to_string(), "ALU ADD");
-        assert_eq!(Instruction::Alu(AluOperation::Sub).display(regs).to_string(), "ALU SUB");
-        assert_eq!(Instruction::Alu(AluOperation::AddC).display(regs).to_string(), "ALU ADDC");
-        assert_eq!(Instruction::Alu(AluOperation::SubC).display(regs).to_string(), "ALU SUBC");
-        assert_eq!(Instruction::Alu(AluOperation::Or).display(regs).to_string(), "ALU OR");
-        assert_eq!(Instruction::Alu(AluOperation::Xor).display(regs).to_string(), "ALU XOR");
-        assert_eq!(Instruction::Alu(AluOperation::And).display(regs).to_string(), "ALU AND");
-        assert_eq!(Instruction::Alu(AluOperation::Not).display(regs).to_string(), "ALU NOT");
-    }
-}
-
-#[test]
-fn alu_valid_shift_or_rotate() {
-    for regs in &[GeneralPurposeRegister::defaults(), alt_gp_registers()] {
-        for &d in &[AluOperationShiftOrRotateDirection::Right, AluOperationShiftOrRotateDirection::Left] {
-            for &tt in &[AluOperationShiftOrRotateType::Lsf,
-                         AluOperationShiftOrRotateType::Asf,
-                         AluOperationShiftOrRotateType::Rtc,
-                         AluOperationShiftOrRotateType::Rtw] {
-                assert_eq!(Instruction::Alu(AluOperation::ShiftOrRotate { d: d, tt: tt }).display(regs).to_string(),
-                           format!("ALU SOR {} {}", d, tt));
-            }
-        }
-    }
-}
-
-#[test]
-fn move_() {
-    for regs in &[GeneralPurposeRegister::defaults(), alt_gp_registers()] {
-        for aaa in regs {
-            for bbb in regs {
-                assert_eq!(Instruction::Move {
-                                   aaa: aaa.address(),
-                                   bbb: bbb.address(),
-                               }
-                               .display(regs)
-                               .to_string(),
-                           format!("MOVE {} {}", aaa.letter(), bbb.letter()));
-            }
-        }
-    }
-}
-
 #[test]
 fn port() {
     single_register("PORT IN", |r| {
         Instruction::Port {
             d: InstructionPortDirection::In,
-            aaa: r,
+            rrr: r,
         }
     });
     single_register("PORT OUT", |r| {
         Instruction::Port {
             d: InstructionPortDirection::Out,
-            aaa: r,
+            rrr: r,
         }
     });
 }
 
 #[test]
 fn comp() {
-    single_register("COMP", |r| Instruction::Comp { aaa: r });
+    single_register("COMP", |r| Instruction::Comp { rrr: r });
 }
 
 #[test]
@@ -191,26 +206,36 @@ fn halt() {
 
 #[test]
 fn reserved_block_0() {
-    reserved_block(0b0000_0000, 0b1111);
+    reserved_block(0b0001_0100, 0b11);
 }
 
 #[test]
 fn reserved_block_1() {
-    reserved_block(0b1000_0000, 0b11_1111);
+    reserved_block(0b0001_1000, 0b111);
 }
 
 #[test]
 fn reserved_block_2() {
-    reserved_block(0b1100_0000, 0b1_1111);
+    reserved_block(0b1000_0000, 0b111111);
 }
 
 #[test]
 fn reserved_block_3() {
-    reserved_block(0b1110_0000, 0b1111);
+    reserved_block(0b1100_0000, 0b1111);
 }
 
 #[test]
 fn reserved_block_4() {
+    reserved_block(0b1101_0000, 0b111);
+}
+
+#[test]
+fn reserved_block_5() {
+    reserved_block(0b1101_1100, 0b11);
+}
+
+#[test]
+fn reserved_block_6() {
     reserved_block(0b1111_1100, 0b1);
 }
 
