@@ -1,36 +1,35 @@
-use pir_8_emu::isa::instruction::{AluOperationShiftOrRotateDirection, AluOperationShiftOrRotateType, InstructionJumpCondition, InstructionPortDirection,
-                                  InstructionMadrDirection, InstructionStckDirection, InstructionRegisterPair, AluOperation, Instruction};
+use pir_8_emu::isa::instruction::{InstructionLoadImmediateWideRegisterPair, AluOperationShiftOrRotateDirection, AluOperationShiftOrRotateType,
+                                  InstructionJumpCondition, InstructionPortDirection, InstructionMadrDirection, InstructionStckDirection,
+                                  InstructionRegisterPair, AluOperation, Instruction};
 use pir_8_emu::isa::GeneralPurposeRegister;
 use self::super::super::alt_gp_registers;
 use std::convert::TryFrom;
 
 
 #[test]
-fn madr() {
+fn load_immediate_byte() {
+    rrr("LOAD IMM BYTE", |rrr| Instruction::LoadImmediateByte { rrr: rrr.address() });
+}
+
+#[test]
+fn load_indirect() {
+    rrr("LOAD IND", |rrr| Instruction::LoadIndirect { rrr: rrr.address() });
+}
+
+#[test]
+fn load_immediate_wide() {
     for regs in &[GeneralPurposeRegister::defaults(), alt_gp_registers()] {
-        assert_eq!(Instruction::from_str("MADR WRITE A&B", regs),
-                   Ok(Instruction::Madr {
-                       d: InstructionMadrDirection::Write,
-                       r: InstructionRegisterPair::Ab,
-                   }));
+        assert_eq!(Instruction::from_str("LOAD IMM WIDE A&B", regs),
+                   Ok(Instruction::LoadImmediateWide { rr: InstructionLoadImmediateWideRegisterPair::Ab }));
 
-        assert_eq!(Instruction::from_str("MADR WRITE C&D", regs),
-                   Ok(Instruction::Madr {
-                       d: InstructionMadrDirection::Write,
-                       r: InstructionRegisterPair::Cd,
-                   }));
+        assert_eq!(Instruction::from_str("LOAD IMM WIDE C&D", regs),
+                   Ok(Instruction::LoadImmediateWide { rr: InstructionLoadImmediateWideRegisterPair::Cd }));
 
-        assert_eq!(Instruction::from_str("MADR READ A&B", regs),
-                   Ok(Instruction::Madr {
-                       d: InstructionMadrDirection::Read,
-                       r: InstructionRegisterPair::Ab,
-                   }));
+        assert_eq!(Instruction::from_str("LOAD IMM WIDE X&Y", regs),
+                   Ok(Instruction::LoadImmediateWide { rr: InstructionLoadImmediateWideRegisterPair::Xy }));
 
-        assert_eq!(Instruction::from_str("MADR READ C&D", regs),
-                   Ok(Instruction::Madr {
-                       d: InstructionMadrDirection::Read,
-                       r: InstructionRegisterPair::Cd,
-                   }));
+        assert_eq!(Instruction::from_str("LOAD IMM WIDE ADR", regs),
+                   Ok(Instruction::LoadImmediateWide { rr: InstructionLoadImmediateWideRegisterPair::Adr }));
     }
 }
 
@@ -61,19 +60,9 @@ fn jump() {
     }
 }
 
-#[test]
-fn load_immediate() {
-    aaa("LOAD IMM", |aaa| Instruction::LoadImmediate { aaa: aaa.address() });
-}
-
-#[test]
-fn load_indirect() {
-    aaa("LOAD IND", |aaa| Instruction::LoadIndirect { aaa: aaa.address() });
-}
-
 #[test]
 fn save() {
-    aaa("SAVE", |aaa| Instruction::Save { aaa: aaa.address() });
+    rrr("SAVE", |rrr| Instruction::Save { rrr: rrr.address() });
 }
 
 #[test]
@@ -156,38 +145,67 @@ fn alu() {
 #[test]
 fn move_() {
     for regs in &[GeneralPurposeRegister::defaults(), alt_gp_registers()] {
-        for aaa in regs {
-            for bbb in regs {
-                assert_eq!(Instruction::from_str(&format!("MOVE {} {}", aaa.letter(), bbb.letter()), regs),
+        for qqq in regs {
+            for rrr in regs {
+                assert_eq!(Instruction::from_str(&format!("MOVE {} {}", qqq.letter(), rrr.letter()), regs),
                            Ok(Instruction::Move {
-                               aaa: aaa.address(),
-                               bbb: bbb.address(),
+                               qqq: qqq.address(),
+                               rrr: rrr.address(),
                            }));
             }
         }
     }
 }
 
+#[test]
+fn madr() {
+    for regs in &[GeneralPurposeRegister::defaults(), alt_gp_registers()] {
+        assert_eq!(Instruction::from_str("MADR WRITE A&B", regs),
+                   Ok(Instruction::Madr {
+                       d: InstructionMadrDirection::Write,
+                       r: InstructionRegisterPair::Ab,
+                   }));
+
+        assert_eq!(Instruction::from_str("MADR WRITE C&D", regs),
+                   Ok(Instruction::Madr {
+                       d: InstructionMadrDirection::Write,
+                       r: InstructionRegisterPair::Cd,
+                   }));
+
+        assert_eq!(Instruction::from_str("MADR READ A&B", regs),
+                   Ok(Instruction::Madr {
+                       d: InstructionMadrDirection::Read,
+                       r: InstructionRegisterPair::Ab,
+                   }));
+
+        assert_eq!(Instruction::from_str("MADR READ C&D", regs),
+                   Ok(Instruction::Madr {
+                       d: InstructionMadrDirection::Read,
+                       r: InstructionRegisterPair::Cd,
+                   }));
+    }
+}
+
 #[test]
 fn port() {
-    aaa("PORT IN", |aaa| {
+    rrr("PORT IN", |rrr| {
         Instruction::Port {
             d: InstructionPortDirection::In,
-            aaa: aaa.address(),
+            rrr: rrr.address(),
         }
     });
 
-    aaa("PORT OUT", |aaa| {
+    rrr("PORT OUT", |rrr| {
         Instruction::Port {
             d: InstructionPortDirection::Out,
-            aaa: aaa.address(),
+            rrr: rrr.address(),
         }
     });
 }
 
 #[test]
 fn comp() {
-    aaa("COMP", |aaa| Instruction::Comp { aaa: aaa.address() });
+    rrr("COMP", |rrr| Instruction::Comp { rrr: rrr.address() });
 }
 
 #[test]
@@ -234,10 +252,10 @@ fn halt() {
 }
 
 
-fn aaa(base: &str, ins: fn(&GeneralPurposeRegister) -> Instruction) {
+fn rrr(base: &str, ins: fn(&GeneralPurposeRegister) -> Instruction) {
     for regs in &[GeneralPurposeRegister::defaults(), alt_gp_registers()] {
-        for aaa in regs {
-            assert_eq!(Instruction::from_str(&format!("{} {}", base, aaa.letter()), regs), Ok(ins(aaa)));
+        for rrr in regs {
+            assert_eq!(Instruction::from_str(&format!("{} {}", base, rrr.letter()), regs), Ok(ins(rrr)));
         }
     }
 }
